/*
 * Copyright 2013, Broadcom Corporation
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;
 * the contents of this file may not be disclosed to third parties, copied
 * or duplicated in any form, in whole or in part, without the prior
 * written permission of Broadcom Corporation.
 */

/** @file
*
* WICED Smart UART Upgrade 
*
* This file provides function required to support WICED Smart Upgrade over the
* Peripheral UART.
*
* To download host sends command to download with length of the patch to be
* transmitted.  After device acks that, host sends fixed chunks of data
* each of each has to be acked.  After all the bytes has been downloaded
* and acknowledged host sends verify command that includes CRC32 of the
* whole patch.  During the download device saves data directly to the EEPROM
* or serial flash.  At the verification stage device reads data back from the
* NVRAM and calculates checksum of the data stored there.  Result of the
* verification is sent back to the host.
*
*/

#include "bleprofile.h"
#include "bleapp.h"
#include "puart.h"
#include "gpiodriver.h"
#include "string.h"
#include "stdio.h"
#include "PUART_Funcs.h"
#include "devicelpm.h"
#include "spar_utils.h"
#include "platform.h"

/******************************************************
 *                      Constants
 ******************************************************/

/******************************************************
 *               Function Prototypes
 ******************************************************/

extern void (*puart_rx_callback)(void*);
UINT32 Puart_device_lpm_queriable(LowPowerModePollType type, UINT32 context);

/******************************************************
 *               Variables Definitions
 ******************************************************/

// Initialize peripheral UART
int App_Puart_init(BLE_PROFILE_PUART_CFG *puart_cfg, void (*puart_rx_callback)(void*))
{

	//For some reason the extern puart_UartConfig declaration below is necessary to get the PUART up and running
	//Will look into this...
	extern puart_UartConfig puart_config;
	puart_config.baudrate = 115200;

	// Select the uart pins for RXD, TXD and optionally CTS and RTS.
	// If hardware flow control is not required like here, set these
	// pins to 0x00. See Table 1 and Table 2 for valid options.
	puart_selectUartPads(33, 32, 00, 00);

	//ble_trace1("Puart_cfg Rx= %d\n     ", puart_cfg->rxpin);
	//ble_trace1("Puart_cfg Tx= %d\n     ", puart_cfg->txpin);
	// Initialize the peripheral uart driver
	puart_init();
	// Since we are not configuring CTS and RTS here, turn off
	// hardware flow control. If HW flow control is used, then
	// puart_flowOff should not be invoked.
	puart_flowOff();

	devlpm_init();
	// Since we are not using any flow control, disable sleep when download starts.
	// If HW flow control is configured or app uses its own flow control mechanism,
	// this is not required.
	devlpm_registerForLowPowerQueries(Puart_device_lpm_queriable, 0);

	// BEGIN - puart interrupt
	//  The following lines enable interrupt when one (or more) bytes
	//  are received over the peripheral uart interface. This is optional.
	//  In the absense of this, the app is expected to poll the peripheral
	//  uart to pull out received bytes.
	// clear interrupt
	P_UART_INT_CLEAR(P_UART_ISR_RX_AFF_MASK);

	// set watermark to 15 byte - will interrupt after 15bytes have been received.
	P_UART_WATER_MARK_RX_LEVEL (1);

	// enable UART interrupt in the Main Interrupt Controller and RX Almost Full in the UART
	// Interrupt Controller
	P_UART_INT_ENABLE |= P_UART_ISR_RX_AFF_MASK;

	// Set callback function to app callback function.
	// Callback function is assigned to whatever Function was passed into App_Puart_Init, in this case 'puart_interrupt_callback'
	puart_rxCb = puart_rx_callback;

	// Enable the CPU level interrupt
	puart_enableInterrupt();

	/* END - puart interrupt */
	return (TRUE);
}

// Send out a stream of bytes to the Peer Device on other end of P_UART
// Can use USB-UART to display data on a terminal window
// buffer - The beffer to send to the Peer
// length - The number of bytes that were sent
// Returns The number of bytes that were sent
UINT32 puart_send_bytes(UINT8* buffer, UINT32 length)
{
	UINT32 ok = length;
	ble_trace0("puart_send_bytes\n");

	// Need to send at least 1 byte.
	if(!buffer || ! length)
		return 0;
	// Write out all the given bytes synchronously
	// If the number of bytes is > P_UART_TX_FIFIO_SIZE(16)
	//puart_write() will block until there is space in the HW FIFO.
	while( length > 0 )
	{
		ble_trace1("P_buffer:  %d\n", *buffer);
		puart_write(*buffer++);
		puart_write(0);
		length--;
	}
	// ble_trace1("Ack:%d", status);

	puart_write(0xA);	//Line Feed
	puart_write(0xD);	//Carriage Return


    return ok;
}

// Callback called by the FW when ready to sleep/deep-sleep. Disable both by returning 0
// when there is an active download ongoing.
UINT32 Puart_device_lpm_queriable(LowPowerModePollType type, UINT32 context)
{
	// Disable sleep.
	return 0;
}
